/*****************************************************************************/
/* main.c : Module principal et ligne de commande
 * main.c : Main module and command line
 *
 *
 * hOpla : Lien XML-SQL - XML-SQL link
 * Copyright (c) 2000 Philippe Roy
 * Auteur - Author : Philippe Roy
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <locale.h>
#include <gnome.h>
#include <dlfcn.h>

#include "main.h"
#include "db2xml.h"
#include "xml2struc.h"
#include "xml2data.h"
#include "admin.h"


/*****************************************************************************/
/* Variables globales */
/*****************************************************************************/
int apropos_mode = FALSE;


/*****************************************************************************/
/* Popt : arguments ligne de commande */
/*****************************************************************************/
const char **arguments;
poptContext pctx;
struct poptOption parametres[] ={
  {
    "about", 0, POPT_ARG_NONE, &apropos_mode, 0,
    N_("About hOpla"), NULL
  },
  {
    "interface", 'i', POPT_ARG_STRING, &HOPLA_INTERFACE_POPT, 0,
    N_("Specify the database's interface : pgsql (PostgreSQL) (obligatory)"), N_("INTERFACE"),
  },
  {
    "host", 'h', POPT_ARG_STRING, &HOPLA_HOTE_POPT, 0,
    N_("Specify the host (SQL connection) (optional)"), N_("HOST"),
  },
  {
    "port", 'p', POPT_ARG_STRING, &HOPLA_PORT_POPT, 0,
    N_("Specify the port (SQL connection) (optional)"), N_("PORT"),
  },
  {
    "user", 'u', POPT_ARG_STRING, &HOPLA_USER_POPT, 0,
    N_("Specify the port (SQL connection) (optional)"), N_("PORT"),
  },
  {
    "password", 'd', POPT_ARG_STRING, &HOPLA_PASSWORD_POPT, 0,
    N_("Specify the user's password (SQL connection) (optional)"), N_("PASSWORD"),
  },
  {
    "base", 'b', POPT_ARG_STRING, &HOPLA_BASE_POPT, 0,
    N_("Specify the database (SQL connection) (obligatory for struc2xml, xml2struc, data2xml or xml2data)"), N_("DATA BASE"),
  },
  {
    "file", 'f', POPT_ARG_STRING, &HOPLA_FILE_POPT, 0,
    N_("Specify the data XML used for import's source or export's target (obligatory)"), N_("FILE"),
  },
  {
    "operation", 'o', POPT_ARG_STRING, &HOPLA_OPERATION_POPT, 0,
    N_("Specify the operation type between the SQL database and the XML file : struc2xml (export structure : base to file), xml2struc (import structure : file to base), data2xml (export data : base to file), xml2data (import data : file to base), db2xml (export structure & data with template : template + base to file), admin2xml (export administration infos : base to file) or xml2admin (import administration infos : file to base) (obligatory)"), N_("OPERATION"),
  },
  {
    "method", 'm', POPT_ARG_STRING, &HOPLA_REFRESH_METHOD_POPT, 0,
    N_("Specify the refresh method : add (add items), remove (remove items) or update (change modified, add new and remove obsolted) (obligatory for xml2data)"), N_("METHOD"),
  },
  {
    "query", 'q', POPT_ARG_STRING, &HOPLA_QUERY_POPT, 0,
    N_("Specify the SQL query (obligatory for data2xml)"), N_("QUERY"),
  },
  {
    "table", 't', POPT_ARG_STRING, &HOPLA_TABLE_POPT, 0,
    N_("Specify the table of query (obligatory for data2xml)"), N_("TABLE"),
  },
  {
    "template", 'g', POPT_ARG_STRING, &HOPLA_TEMPLATE_POPT, 0,
    N_("Specify the template (XML file) (optional for db2xml)"), N_("TEMPLATE"),
  },
  {
    "obso", 0, POPT_ARG_NONE, &HOPLA_OBSO_MODE_POPT, 0,
    N_("obsolete mode : export all data (with obsoleted) (optional for db2xml)"), NULL
  },
  {
    "zip", 0, POPT_ARG_NONE, &HOPLA_ZIP_MODE_POPT, 0,
    N_("zip mode : compress the XML output file (optional for struc2xml, data2xml or admin2xml)"), NULL
  },
  {
    "test", 0, POPT_ARG_NONE, &HOPLA_TEST_MODE_POPT, 0,
    N_("test mode : only test XML input file (optional for xml2struc, xml2data or xml2admin)"), NULL
  },
  {
    "log", 'l', POPT_ARG_STRING, &HOPLA_XMLLOG_FILE_POPT, 0,
    N_("generate a XML log file (optional)"), N_("LOG FILE"),
  },
  {
    "nocolor", 0, POPT_ARG_NONE, &HOPLA_NOCOLOR_MODE_POPT, 0,
    N_("no colors mode : echo without colors"), NULL
  },
  {
    NULL, '\0', 0, NULL, 0, 
    NULL, NULL
  }
};


/*****************************************************************************/
/* Main */
/*****************************************************************************/
int main (int argc, char *argv[])
{
  gchar *message = NULL;
  gchar *txt_tampon = NULL;
  void (*plugins_login)();
  void (*plugins_logout)();
  void *greffon_tampon;

  /* Init du popt */
  HOPLA_HOTE_POPT = NULL;
  HOPLA_PORT_POPT = NULL;
  HOPLA_USER_POPT = NULL;
  HOPLA_PASSWORD_POPT = NULL;
  HOPLA_BASE_POPT = NULL;
  HOPLA_FILE_POPT = NULL;
  HOPLA_OPERATION_POPT = NULL;
  HOPLA_QUERY_POPT = NULL;
  HOPLA_TABLE_POPT = NULL;
  HOPLA_REFRESH_METHOD_POPT = NULL;
  HOPLA_TEMPLATE_POPT = NULL;
  HOPLA_OBSO_MODE_POPT = FALSE;
  HOPLA_ZIP_MODE_POPT = FALSE;
  HOPLA_TEST_MODE_POPT = FALSE;
  HOPLA_XMLLOG_FILE_POPT = NULL;

  /* Initialisation */
  bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain (PACKAGE);
  gnome_init_with_popt_table ("hopla", VERSION, argc, argv, parametres, 0, &pctx);
  arguments = poptGetArgs (pctx);
  printf ("hOpla - %s\n", VERSION);
  if (HOPLA_XMLLOG_FILE_POPT)
    hopla_open_xmllog ();

  /* A propos */
  if (apropos_mode)
    {
      printf (_("hOpla - %s : XML-SQL link\n"),VERSION);
      printf ("Copyright (c) 2000 Philippe Roy\n");
      printf (_("Author : Philippe Roy\n"));
      printf (_("hOpla is a free software, covered by the GNU General Public License.\n"));
      printf (_("http://hopla.sourceforge.net\n"));
      printf ("\n");
      printf (_("\"hOpla\" rather \"Hopla !\" is an Alsatian expression (French dialect , speaked in Alsace (Strasbourg, Colmar, Mulhouse, ...).\n"));
      printf (_("It means generally \"Let's go !\". His use is extremely frequent indeed all-occasion and amoung others things before to clink glasses.\n"));
      printf (_("So hOpla provide the logistic flow between the barret (a littl' Gewurtz late vintage, for exemple) (SQL database) and the glasses (for white wine in Gewurt case) (XML files).\n"));
      return 0;
    }

  /* Vrif que interface, fichier, base et opration <> null */
  if (!HOPLA_INTERFACE_POPT)
    hopla_message (_("no interface specified"), "stop");
  if (!HOPLA_FILE_POPT)
    hopla_message (_("no file specified"), "stop");
  if (!HOPLA_OPERATION_POPT)
    hopla_message (_("no operation specified"), "stop");

  /* Vrif de l'interface */
  HOPLA_PLUGINS_DIRECTORY = g_strdup_printf ("%s/lib/hopla/plugins", PACKAGE_DATA_DIR);
  if (!strcmp (HOPLA_INTERFACE_POPT, "pgsql"))
    HOPLA_PLUGIN_FILE = g_strdup_printf ("%s/libpgsql.so", HOPLA_PLUGINS_DIRECTORY);
  else
    hopla_message (_("unknow interface"), "stop");

  /* Vrif que l'argument opration est conforme */
  if (!((!strcmp (HOPLA_OPERATION_POPT, "struc2xml")) || 
	(!strcmp (HOPLA_OPERATION_POPT, "xml2struc")) || 
	(!strcmp (HOPLA_OPERATION_POPT,"data2xml")) || 
	(!strcmp (HOPLA_OPERATION_POPT,"db2xml")) || 
	(!strcmp (HOPLA_OPERATION_POPT, "xml2data")) || 
	(!strcmp (HOPLA_OPERATION_POPT, "admin2xml")) || 
	(!strcmp (HOPLA_OPERATION_POPT, "xml2admin")))) 
    hopla_message (_("unknow operation"), "stop");

  /* Vrif de la base */
  if ((!strcmp (HOPLA_OPERATION_POPT, "xml2admin")) || (!strcmp (HOPLA_OPERATION_POPT, "admin2xml")))
    {
      if (HOPLA_BASE_POPT)
	hopla_message (_("don't specified the base for administration operation"), "stop");
    }
    else
      if (!HOPLA_BASE_POPT)
	hopla_message (_("no base specified"), "stop");

  /* Vrif que la requte SQL figure bien pour l'opration data2xml */
  if ((!strcmp (HOPLA_OPERATION_POPT, "data2xml")) && (!HOPLA_QUERY_POPT))
    hopla_message (_("no query specified for data2xml operation"), "stop");
  if ((!strcmp (HOPLA_OPERATION_POPT, "data2xml")) && (!HOPLA_TABLE_POPT))
    hopla_message (_("no query's table specified for data2xml operation"), "stop");

  /* Vrif que l'argument mthode de mise  jour est conforme pour xml2data */
  if ((!strcmp (HOPLA_OPERATION_POPT, "xml2data")) && (!HOPLA_REFRESH_METHOD_POPT))
    hopla_message (_("no refresh method specified for xml2data operation"), "stop");
  if (HOPLA_REFRESH_METHOD_POPT)
    if (!((!strcmp (HOPLA_REFRESH_METHOD_POPT, "add")) || (!strcmp (HOPLA_REFRESH_METHOD_POPT, "remove")) || (!strcmp (HOPLA_REFRESH_METHOD_POPT, "update"))))
      hopla_message (_("unknow refresh method"), "stop");

  /* Vrif que le mode zip est dsactiv pour xml2struc, xml2data et xml2admin, car inutile */
  if ((HOPLA_ZIP_MODE_POPT) && ((!strcmp (HOPLA_OPERATION_POPT, "xml2struc")) || (!strcmp (HOPLA_OPERATION_POPT, "xml2data")) || (!strcmp (HOPLA_OPERATION_POPT, "xml2admin"))))
    hopla_message (_("zip mode need unenable for xml2struc operation or xml2data operation"), "stop");

  /* Vrif que le mode test est dsactiv pour struc2xml, data2xml et admin2xml, car inutile */
  if ((HOPLA_TEST_MODE_POPT) && ((!strcmp (HOPLA_OPERATION_POPT, "struc2xml")) || (!strcmp (HOPLA_OPERATION_POPT, "data2xml")) || (!strcmp (HOPLA_OPERATION_POPT, "admin2xml"))))
    hopla_message (_("test mode need unenable for struc2xml operation or data2xml operation"), "stop");

  /* Vrif que le mode obso est dsactiv pour les autres que data2xml, car inutile */
  if ((HOPLA_OBSO_MODE_POPT) && (strcmp (HOPLA_OPERATION_POPT, "data2xml")))
    hopla_message (_("obso mode need unenable for others operations than data2xml operation"), "stop");

  /* Go ! */
  if (!strcmp (HOPLA_OPERATION_POPT, "struc2xml"))
    if (!struc2xml_main ())
      hopla_message (g_strdup_printf ("%s %s %s" , "struc2xml", _("operation"), _("failed")), "failed");
  if (!strcmp (HOPLA_OPERATION_POPT, "xml2struc"))
    if (!xml2struc_main ())
      hopla_message (g_strdup_printf ("%s %s %s" , "xml2struc", _("operation"), _("failed")), "failed");
  if (!strcmp (HOPLA_OPERATION_POPT, "data2xml"))
    if (!data2xml_main ())
      hopla_message (g_strdup_printf ("%s %s %s" , "data2xml", _("operation"), _("failed")), "failed");
  if (!strcmp (HOPLA_OPERATION_POPT, "db2xml"))
    if (!db2xml_main ())
      hopla_message (g_strdup_printf ("%s %s %s" , "db2xml", _("operation"), _("failed")), "failed");
  if (!strcmp (HOPLA_OPERATION_POPT, "xml2data"))
    if (!xml2data_main ())
      hopla_message (g_strdup_printf ("%s %s %s" , "xml2data", _("operation"), _("failed")), "failed");
  if (!strcmp (HOPLA_OPERATION_POPT, "xml2admin"))
    if (!xml2admin_main ())
      hopla_message (g_strdup_printf ("%s %s %s" , "xml2admin", _("operation"), _("failed")), "failed");
  if (!strcmp (HOPLA_OPERATION_POPT, "admin2xml"))
    if (!admin2xml_main ())
      hopla_message (g_strdup_printf ("%s %s %s" , "admin2xml", _("operation"), _("failed")), "failed");
  if (HOPLA_XMLLOG_FILE_POPT)
    hopla_close_xmllog ();
  printf ("hOpla : %s\n", _("Bye"));
  return 0;
}
