/*****************************************************************************/
/* xml2data.c : Importe les donnes d'un fichier vers la base de donnes
 * xml2data.c : Import file's data to database
 *
 *
 * hOpla : Lien XML-SQL - XML-SQL link
 * Copyright (c) 2000 Philippe Roy
 * Auteur - Author : Philippe Roy
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "main.h"
#include "xml2struc.h"
#include "xml2data.h"


/*****************************************************************************/
/* Test de conformit du fichier */
/*****************************************************************************/
gboolean check_compatibility (Td_xmlbase old_base, Td_xmlbase new_base)
{
  Td_xmltable old_table;
  Td_xmltable new_table;
  Td_xmlfield old_field;
  Td_xmlfield new_field;
  int i, j, k;
  gboolean erreur;
  gboolean erreur_type;
  gchar *txt_tampon = NULL;

  /* Table : nom */
  for (k=0; k < new_base->nbtables; k++)
    {
      new_table = new_base->tables[k];
      txt_tampon = g_strdup_printf (_("Checking compatibility : '%s' (table) defined in database ? ..."), new_table->name);
      erreur = TRUE;
      for (i=0; i < old_base->nbtables; i++)
	{
	  old_table = old_base->tables[i];
	  if (!strcmp (old_table->name, new_table->name))
	    erreur = FALSE;
	}
      if (erreur)
	{
	  hopla_message (txt_tampon, "failed");
	  return FALSE;
	}
      hopla_message (txt_tampon, "done");
    }

  /* Correspondance base et fichier sur les champs et types */
  for (k=0; k < new_base->nbtables; k++)
    {
      new_table = new_base->tables[k];
      i =0;
      do
	{
	  old_table = old_base->tables[i];
	  i++;
	}
      while (strcmp (old_table->name, new_table->name));
      for (i=0; i < new_table->nbfields; i++)
	{
	  erreur = TRUE;
	  erreur_type = TRUE;
	  new_field = new_table->fields[i];
	  for (j=0; j < old_table->nbfields; j++)
	    {
	      old_field = old_table->fields[j];
	      if (!strcmp (old_field->name, new_field->name))
		erreur = FALSE;
	      if ((!strcmp (old_field->name, new_field->name))&&(!strcmp (old_field->type, new_field->type)))
		erreur_type = FALSE;
	    }
	  txt_tampon = g_strdup_printf (_("Checking compatibility : '%s.%s' (field) defined in database ? ..."), new_table->name, new_field->name);
	  if (erreur)
	    {
	      hopla_message (txt_tampon, "failed");
	      return FALSE;
	    }
	  hopla_message (txt_tampon, "done");
	  txt_tampon = g_strdup_printf (_("Checking compatibility : '%s.%s.%s' (type) defined in database ? ..."), new_table->name, new_field->name, new_field->type);
	  if (erreur_type)
	    {
	      hopla_message (txt_tampon, "failed");
	      return FALSE;
	    }
	  hopla_message (txt_tampon, "done");
	}
    }
  return TRUE;
}


/*****************************************************************************/
/* Rafraichir les donnes */
/*****************************************************************************/
Td_xmlbase synchronize (Td_xmlbase old_base, Td_xmlbase new_base)
{
  Td_xmltable old_table;
  Td_xmltable new_table;
  Td_xmlfield old_field;
  Td_xmlfield new_field;
  Td_xmldatatable old_datatable;
  Td_xmldatatable new_datatable;
  int new_index_hopla_id;
  int new_index_hopla_id_parent;
  int new_index_hopla_id_sibling;
  int new_index_hopla_obso;
  int old_index_hopla_id;
  int old_index_hopla_id_parent;
  int old_index_hopla_id_sibling;
  int old_index_hopla_obso;
  int i, j, k, m;
  int cur_id;
  int relation_id[500][2];
  int relation_field[50];
  gchar *requete;
  gchar *txt_tampon;
  gboolean bool_tampon;


  /* Synchronisation de la base */
  for (i=0; i < new_base->nbdatatables; i++)
    {
      new_datatable = new_base->datatables[i];

      /* Recherche de la table source dans le fichier */
      txt_tampon = g_strdup_printf (_("Searching table '%s' in file's structure"), new_datatable->table_source);
      j = hopla_findindex_table (new_base, new_datatable->table_source);
      if (j == -1)
	{
	  hopla_message (txt_tampon, "failed");
	  return NULL;
	}
      new_table = new_base->tables[j];
      hopla_message (txt_tampon, "done");

      /* Recherche de la table source dans la base de donnes */
      txt_tampon = g_strdup_printf (_("Searching table '%s' in database's structure"), new_datatable->table_source);
      j = hopla_findindex_table (old_base, new_datatable->table_source);
      if (j == -1)
	{
	  hopla_message (txt_tampon, "failed");
	  return NULL;
	}
      old_table = old_base->tables[j];
      hopla_message (txt_tampon, "done");

      /* Recherche des champs administrateurs dans le fichier */
      txt_tampon = g_strdup_printf (_("Searching administation field '%s' in file's structure"), "hopla_id");
      new_index_hopla_id = hopla_findindex_field (new_table, "hopla_id");
      if (new_index_hopla_id == -1)
	{
	  hopla_message (txt_tampon, "failed");
	  return NULL;
	}
      txt_tampon = g_strdup_printf ("%s : field #%d", txt_tampon, new_index_hopla_id);
      hopla_message (txt_tampon, "done");
      txt_tampon = g_strdup_printf (_("Searching administation field '%s' in file's structure"), "hopla_id_parent");
      new_index_hopla_id_parent = hopla_findindex_field (new_table, "hopla_id_parent");
      if (new_index_hopla_id_parent == -1)
	{
	  hopla_message (txt_tampon, "failed");
	  return NULL;
	}
      txt_tampon = g_strdup_printf ("%s : field #%d", txt_tampon, new_index_hopla_id_parent);
      hopla_message (txt_tampon, "done");
      txt_tampon = g_strdup_printf (_("Searching administation field '%s' in file's structure"), "hopla_id_sibling");
      new_index_hopla_id_sibling = hopla_findindex_field (new_table, "hopla_id_sibling");
      if (new_index_hopla_id_sibling == -1)
	{
	  hopla_message (txt_tampon, "failed");
	  return NULL;
	}
      txt_tampon = g_strdup_printf ("%s : field #%d", txt_tampon, new_index_hopla_id_sibling);
      hopla_message (txt_tampon, "done");
      txt_tampon = g_strdup_printf (_("Searching administation field '%s' in file's structure"), "hopla_obso");
      new_index_hopla_obso = hopla_findindex_field (new_table, "hopla_obso");
      if (new_index_hopla_obso == -1)
	{
	  hopla_message (txt_tampon, "failed");
	  return NULL;
	}
      txt_tampon = g_strdup_printf ("%s : field #%d", txt_tampon, new_index_hopla_obso);
      hopla_message (txt_tampon, "done");

      /* Recherche des champs administrateurs dans la base de donnes */
      txt_tampon = g_strdup_printf (_("Searching administation field '%s' in database's structure"), "hopla_id");
      old_index_hopla_id = hopla_findindex_field (old_table, "hopla_id");
      if (old_index_hopla_id == -1)
	{
	  hopla_message (txt_tampon, "failed");
	  return NULL;
	}
      txt_tampon = g_strdup_printf ("%s : field #%d", txt_tampon, old_index_hopla_id);
      hopla_message (txt_tampon, "done");
      txt_tampon = g_strdup_printf (_("Searching administation field '%s' in database's structure"), "hopla_id_parent");
      old_index_hopla_id_parent = hopla_findindex_field (old_table, "hopla_id_parent");
      if (old_index_hopla_id_parent == -1)
	{
	  hopla_message (txt_tampon, "failed");
	  return NULL;
	}
      txt_tampon = g_strdup_printf ("%s : field #%d", txt_tampon, old_index_hopla_id_parent);
      hopla_message (txt_tampon, "done");
      txt_tampon = g_strdup_printf (_("Searching administation field '%s' in database's structure"), "hopla_id_sibling");
      old_index_hopla_id_sibling = hopla_findindex_field (old_table, "hopla_id_sibling");
      if (old_index_hopla_id_sibling == -1)
	{
	  hopla_message (txt_tampon, "failed");
	  return NULL;
	}
      txt_tampon = g_strdup_printf ("%s : field #%d", txt_tampon, old_index_hopla_id_sibling);
      hopla_message (txt_tampon, "done");
      txt_tampon = g_strdup_printf (_("Searching administation field '%s' in database's structure"), "hopla_obso");
      old_index_hopla_obso = hopla_findindex_field (old_table, "hopla_obso");
      if (old_index_hopla_obso == -1)
	{
	  hopla_message (txt_tampon, "failed");
	  return NULL;
	}
      txt_tampon = g_strdup_printf ("%s : field #%d", txt_tampon, old_index_hopla_obso);
      hopla_message (txt_tampon, "done");

      /* Lecture de la table de donnes */
      requete = g_strdup_printf ("SELECT * FROM %s", new_datatable->table_source);
      txt_tampon = g_strdup_printf (_("Reading datatable '%s' in database"), new_datatable->table_source);
      hopla_message (txt_tampon, "begin");
      old_datatable = hopla_query (requete);
      if (!old_datatable)
	{
	  hopla_message (txt_tampon, "failed");
	  hopla_message (txt_tampon, "end");
	  return NULL;
	}
      hopla_message (txt_tampon, "done");
      hopla_message (txt_tampon, "end");

      /* Recherche du pivot */
      cur_id = 0;
      for (j=0; j<old_datatable->nbitems; j++)
	{
	  if (cur_id < atoi (old_datatable->items[j][old_index_hopla_id]))
	    cur_id = atoi (old_datatable->items[j][old_index_hopla_id]);
	}
      txt_tampon = g_strdup_printf ("%s hopla_id #%d", _("Searching the pivot number (number of referenced data in database, before pivot : data exist and after pivot : data is new)"), cur_id);
      hopla_message (txt_tampon, "done");

      /* Affectation de hopla_id (demande de cration de node) et table de correspondance */
      for (j=0; j<new_datatable->nbitems; j++)
	if (0 > atoi (new_datatable->items[j][new_index_hopla_id]))
	  {
	    cur_id++;
	    txt_tampon = g_strdup_printf (_("Synchronizing : creating node : change '%s' : '%s' to '%d'"), "hopla_id", new_datatable->items[j][new_index_hopla_id], cur_id);
	    relation_id[j][0] = atoi (new_datatable->items[j][new_index_hopla_id]);
	    relation_id[j][1] = cur_id;
	    new_datatable->items[j][new_index_hopla_id] = g_strdup_printf ("%d", cur_id);
	    HOPLA_NEW_NODE[i][j] = TRUE;
	    hopla_message (txt_tampon, "done");
	  }
      
      /* Modifier les hopla_id_parent */
      for (j=0; j<new_datatable->nbitems; j++)
	if (0 > atoi (new_datatable->items[j][new_index_hopla_id_parent]))
	  {
	    bool_tampon = TRUE;
	    for (k=0; k<500; k++)
	      if (relation_id[k][0] == atoi (new_datatable->items[j][new_index_hopla_id_parent]))
		{
		  txt_tampon = g_strdup_printf (_("Synchronizing : change '%s' : '%s' to '%d'"), "hopla_id_parent", new_datatable->items[j][new_index_hopla_id_parent], relation_id[k][1]);
		  bool_tampon = FALSE;
		  new_datatable->items[j][new_index_hopla_id_parent] = g_strdup_printf ("%d", relation_id[k][1]);
		  if (!HOPLA_NEW_NODE[i][j])
		    HOPLA_CHANGE_NODE[i][j] = TRUE;
		  hopla_message (txt_tampon, "done");
		  break;
		}
	    if (bool_tampon)
	      {
		txt_tampon = g_strdup_printf (_("Synchronizing : change '%s' : '%s' to ?"), "hopla_id_parent", new_datatable->items[j][new_index_hopla_id_parent]);
		hopla_message (txt_tampon, "failed");
		return NULL;
	      }
	  }

      /* Modifier les hopla_id_sibling */
      for (j=0; j<new_datatable->nbitems; j++)
	if (0 > atoi (new_datatable->items[j][new_index_hopla_id_sibling]))
	  {
	    bool_tampon = TRUE;
	    for (k=0; k<500; k++)
	      if (relation_id[k][0] == atoi (new_datatable->items[j][new_index_hopla_id_sibling]))
		{
		  txt_tampon = g_strdup_printf (_("Synchronizing : change '%s' : '%s' to '%d'"), "hopla_id_buddy", new_datatable->items[j][new_index_hopla_id_sibling], relation_id[k][1]);
		  bool_tampon = FALSE;
		  new_datatable->items[j][new_index_hopla_id_sibling] = g_strdup_printf ("%d", relation_id[k][1]);
		  if (!HOPLA_NEW_NODE[i][j])
		    HOPLA_CHANGE_NODE[i][j] = TRUE;
		  hopla_message (txt_tampon, "done");
		  break;
		}
	    if (bool_tampon)
	      {
		txt_tampon = g_strdup_printf (_("Synchronizing : change '%s' : '%s' to ?"), "hopla_id_parent", new_datatable->items[j][new_index_hopla_id_sibling]);
		hopla_message (txt_tampon, "failed");
		return NULL;
	      }
	  }

      /* Etablir le tableau de correspondance des champs pour comparaison de donnes */
      for (j=0; j<new_table->nbfields; j++)
	{
	  new_field = new_table->fields[j];
	  bool_tampon = TRUE;
	  txt_tampon = g_strdup_printf (_("Searching field '%s' in database's structure"), new_field->name);
	  for (k=0; j<old_table->nbfields; k++)
	    {
	      old_field = old_table->fields[k];
	      if (!strcmp (new_field->name, old_field->name))
		{
		  bool_tampon = FALSE;
		  relation_field[j] = k;
		  txt_tampon = g_strdup_printf ("%s : field #%d", txt_tampon, k);
		  hopla_message (txt_tampon, "done");
		  break;
		}
	    }
	  if (bool_tampon)
	    {
	      hopla_message (txt_tampon, "failed");
	      return NULL;
	    }
	}

      /* Comparaison des donnes et rafrachissement des donnes modifies */
      for (j=0; j<new_datatable->nbitems; j++)
	{
	  if (HOPLA_NEW_NODE[i][j])
	    continue;
	  if (HOPLA_CHANGE_NODE[i][j])
	    continue;
	  txt_tampon = g_strdup_printf (_("Synchronizing : file's hopla_id #%d is changed ?"), j);

	  /* Recherche de la node dans la base de donnes */
	  bool_tampon = TRUE;
	  for (k=0; k<old_datatable->nbitems; k++)
	    if (!strcmp (new_datatable->items[j][new_index_hopla_id], old_datatable->items[k][old_index_hopla_id]))
	      {
		bool_tampon = FALSE;
		break;
	      }
	  if (bool_tampon)
	    {
	      hopla_message (txt_tampon, "failed");
	      return NULL;
	    }

	  /* Comparaison des champs */
	  bool_tampon = TRUE;
	  for (m=0; m<new_table->nbfields; m++)
	    {
	      if (strcmp (new_datatable->items[j][m], old_datatable->items[k][relation_field[m]]))
		{
		bool_tampon = FALSE;
		HOPLA_CHANGE_NODE[i][j] = TRUE;
		hopla_message (txt_tampon, "yes");;
		break;
	      }
	    }
	  if (bool_tampon)
	    {
	      HOPLA_CHANGE_NODE[i][j] = FALSE;
	      hopla_message (txt_tampon, "no");
	    }
	}

      /* Mise en obsolescence */
      for (j=0; j < old_datatable->nbitems; j++)
	if (strcmp (old_datatable->items[j][old_index_hopla_obso], "t"))
	  {
	    bool_tampon=TRUE;
	    txt_tampon = g_strdup_printf (_("Synchronizing : database's hopla_id #%d is obsoleted ?"), j);
	    for (k=0; k < new_datatable->nbitems; k++)
	      if (!strcmp (new_datatable->items[k][new_index_hopla_id], old_datatable->items[j][old_index_hopla_id]))
	      {
		bool_tampon = FALSE;
		HOPLA_OBSO_NODE[i][j] = FALSE;
		hopla_message (txt_tampon, "no");
		break;
	      }
	    if (bool_tampon)
	      {
		HOPLA_OBSO_NODE[i][j] = TRUE;
		hopla_message (txt_tampon, "yes");;
	      }
	  }
    }
  return new_base;
}


/*****************************************************************************/
/* Local main */
/*****************************************************************************/
gboolean xml2data_main (void)
{
  Td_xmlbase new_base;
  Td_xmlbase old_base;
  gchar *txt_tampon = NULL;
  int system_verif;

  /* Lecture du fichier */
  txt_tampon = g_strdup_printf (_("Reading file '%s'"), HOPLA_FILE_POPT);
  hopla_message (txt_tampon, "begin");
  new_base = td_parse_xmlbase (HOPLA_FILE_POPT);
  if (!new_base)
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");

  /* Vrification du fichier */
  txt_tampon = g_strdup_printf ("%s '%s'", _("Checking file"), HOPLA_FILE_POPT);
  hopla_message (txt_tampon, "begin");
  if (!check_name (new_base))
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  if (!check_type (new_base))
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");

  /* Lecture de la base de donnes */
  txt_tampon = g_strdup_printf (_("Reading database '%s'"), HOPLA_BASE_POPT);
  hopla_message (txt_tampon, "begin");
  old_base = hopla_parse_dbbase ();
  if (!old_base)
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");

  /* Vrification du fichier par rapport  la base de donnes */
  txt_tampon = g_strdup_printf (_("Checking file/database compatibility"));
  hopla_message (txt_tampon, "begin");
  if (!check_compatibility (old_base, new_base))
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");

  /* Rafraichir les donnes */
  if (!strcmp (HOPLA_REFRESH_METHOD_POPT, "update"))
    {
      /* Synchronisation des donnes */
      txt_tampon = g_strdup_printf (_("Synchronizing database"));
      hopla_message (txt_tampon, "begin");
      new_base = synchronize (old_base, new_base);
      if (!new_base)
	{
	  hopla_message (txt_tampon, "failed");
	  hopla_message (txt_tampon, "end");
	  return FALSE;
	}
      hopla_message (txt_tampon, "done");
      hopla_message (txt_tampon, "end");

      /* Cration d'une sauvegarde */
      txt_tampon = g_strdup_printf("%s.bak", HOPLA_FILE_POPT);
      system_verif = rename (HOPLA_FILE_POPT, txt_tampon);
      txt_tampon = g_strdup_printf ("Creating backup file : command 'rename %s.bak'", HOPLA_FILE_POPT);
      if (system_verif)
	{
	  hopla_message (txt_tampon, "failed");
	  return FALSE;
	}
      hopla_message (txt_tampon, "done");

      /* Sauvegarde de la nouvelle base */
      txt_tampon = g_strdup_printf (_("Writing file's data (with data updated) '%s'"), HOPLA_FILE_POPT);
      hopla_message (txt_tampon, "begin");
      if (!td_save_xmlbase (HOPLA_FILE_POPT, new_base, "base"))
	{
	  hopla_message (txt_tampon, "failed");
	  hopla_message (txt_tampon, "end");
	  return FALSE;
	}
      hopla_message (txt_tampon, "done");
      hopla_message (txt_tampon, "end");
    }

  /* Ecriture base donnes : table */
  txt_tampon = g_strdup_printf (_("Writting database's data '%s'"), HOPLA_BASE_POPT);
  hopla_message (txt_tampon, "begin");
  if (!hopla_write_dbdatatable (new_base))
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");
  return TRUE;
}
