/*****************************************************************************/
/* db.c : Commandes de bases de donnes
 * db.c : Data base commands
 *
 *
 * hOpla : Lien XML-SQL - XML-SQL link
 * Copyright (c) 2000 Philippe Roy
 * Auteur - Author : Philippe Roy
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <dlfcn.h>

#include "hopla.h"


/*****************************************************************************/
/* Lecture */
/*****************************************************************************/
Td_xmlbase hopla_parse_dbbase (void)
{
  Td_xmlbase ret = NULL;
  Td_xmlbase (*plugins_parse_dbbase)();
  void *greffon_tampon;

  /* Go ! */
  if (!hopla_verif_plugins (HOPLA_PLUGIN_FILE, "plugins_parse_dbbase"))
    return (NULL);
  greffon_tampon = dlopen (HOPLA_PLUGIN_FILE, RTLD_LAZY);
  plugins_parse_dbbase = dlsym (greffon_tampon, "plugins_parse_dbbase");
  ret = plugins_parse_dbbase (NULL);
  if (!ret)
    return (NULL);
  return ret;
}

Td_xmlbase hopla_read_dbbase (Td_xmlbase template)
{
  Td_xmlbase ret = NULL;
  Td_xmlbase (*plugins_read_dbbase)();
  void *greffon_tampon;

  /* Go ! */
  if (!hopla_verif_plugins (HOPLA_PLUGIN_FILE, "plugins_read_dbbase"))
    return (NULL);
  greffon_tampon = dlopen (HOPLA_PLUGIN_FILE, RTLD_LAZY);
  plugins_read_dbbase = dlsym (greffon_tampon, "plugins_read_dbbase");
  ret = plugins_read_dbbase (template);
  if (!ret)
    return (NULL);
  return ret;
}

Td_xmldatatable hopla_query (char *requete)
{
  Td_xmldatatable ret;
  Td_xmldatatable (*plugins_query)();
  void *greffon_tampon;

  /* Go ! */
  if (!hopla_verif_plugins (HOPLA_PLUGIN_FILE, "plugins_query"))
    return (NULL);
  greffon_tampon = dlopen (HOPLA_PLUGIN_FILE, RTLD_LAZY);
  plugins_query = dlsym (greffon_tampon, "plugins_query");
  ret = plugins_query (requete);
  if (!ret)
    return (NULL);
  return ret;
}


/*****************************************************************************/
/* Ecriture */
/*****************************************************************************/
gboolean hopla_write_dbbase (Td_xmlbase new_base)
{
  gboolean (*plugins_write_dbbase)();
  void *greffon_tampon;

  /* Go ! */
  if (!hopla_verif_plugins (HOPLA_PLUGIN_FILE, "plugins_write_dbbase"))
    return FALSE;
  greffon_tampon = dlopen (HOPLA_PLUGIN_FILE, RTLD_LAZY);
  plugins_write_dbbase = dlsym (greffon_tampon, "plugins_write_dbbase");
  if (!plugins_write_dbbase (new_base))
    return FALSE;
  return TRUE;
}

gboolean hopla_write_dbdatatable (Td_xmlbase new_base)
{
  gboolean (*plugins_write_dbdatatable)();
  void *greffon_tampon;

  /* Go ! */
  if (!hopla_verif_plugins (HOPLA_PLUGIN_FILE, "plugins_write_dbdatatable"))
    return FALSE;
  greffon_tampon = dlopen (HOPLA_PLUGIN_FILE, RTLD_LAZY);
  plugins_write_dbdatatable = dlsym (greffon_tampon, "plugins_write_dbdatatable");
  if (!plugins_write_dbdatatable (new_base))
    return FALSE;
  return TRUE;
}


/*****************************************************************************/
/* Adaptation valeur */
/* fr: copie de la GLib : g_strescape */
/* en: GLib copy : g_strescape */
/*****************************************************************************/
gchar *hopla_strescape (const gchar *source)
{
  const guchar *p = (guchar *) source;
  /* Each source byte needs maximally four destination chars (\777) */
  gchar *dest = g_malloc (strlen (source) * 4 + 1);
  gchar *q = dest;
  guchar excmap[256];
  memset (excmap, 0, 256);

  while (*p)
    {
      if (excmap[*p])
	*q++ = *p;
      else
	{
	  switch (*p)
	    {
	    case '\'':
	      *q++ = '\\';
	      *q++ = '\'';
	      break;
	    default:
	      if ((*p < ' ') || (*p >= 0177))
		{
		  *q++ = '\\';
		  *q++ = '0' + (((*p) >> 6) & 07);
		  *q++ = '0' + (((*p) >> 3) & 07);
		  *q++ = '0' + ((*p) & 07);
		}
	      else
		*q++ = *p;
	      break;
	    }
	}
      p++;
    }
  *q = 0;
  return dest;
}


/*****************************************************************************/
/* Recherche d'index */
/*****************************************************************************/
int hopla_findindex_table (Td_xmlbase base, gchar *table_name)
{
  Td_xmltable table;
  int ret;

  /* Go ! */
  for (ret=0; ret < base->nbtables; ret++)
    {
      table = base->tables[ret];
      if (!strcmp (table->name, table_name))
	return ret;
    }
  return -1;
}

int hopla_findindex_field (Td_xmltable table, gchar *field_name)
{
  Td_xmlfield field;
  int ret;

  /* Go ! */
  for (ret=0; ret < table->nbfields; ret++)
    {
      field = table->fields[ret];
      if (!strcmp (field->name, field_name))
	return ret;
    }
  return -1;
}


/*****************************************************************************/
/* Administration */
/*****************************************************************************/
/* Liste des bases disponibles */
Td_xmldatatable hopla_admin_read (void)
{
  Td_xmldatatable ret = NULL;
  Td_xmldatatable (*plugins_admin_read)();
  void *greffon_tampon;

  /* Go ! */
  if (!hopla_verif_plugins (HOPLA_PLUGIN_FILE, "plugins_admin_read"))
    return FALSE;
  greffon_tampon = dlopen (HOPLA_PLUGIN_FILE, RTLD_LAZY);
  plugins_admin_read = dlsym (greffon_tampon, "plugins_admin_read");
  ret = plugins_admin_read (NULL);
  return ret;
}

/* Liste des bases disponibles */
gboolean hopla_admin_write (Td_xmldatatable new_datatable, Td_xmldatatable old_datatable)
{
  gboolean (*plugins_admin_write)();
  void *greffon_tampon;

  /* Go ! */
  if (!hopla_verif_plugins (HOPLA_PLUGIN_FILE, "plugins_admin_write"))
    return FALSE;
  greffon_tampon = dlopen (HOPLA_PLUGIN_FILE, RTLD_LAZY);
  plugins_admin_write = dlsym (greffon_tampon, "plugins_admin_write");
  if (!plugins_admin_write (new_datatable, old_datatable))
    return FALSE;
  return TRUE;
}
