/*****************************************************************************/
/* xml.c : Analyse de fichier XML
 * xml.c : XML parsing
 *
 *
 * ToutDoux : Chtit gestionnaire de projet - A littl' project manage
 * Copyright (c) 2000 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@yahoo.com>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <parser.h>
#include <tree.h>

#include "hopla.h"


/*****************************************************************************/
/* Structure */
/*****************************************************************************/
/* Champs */
Td_xmlfield td_parse_xmlfield (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
{
  Td_xmlfield ret = NULL;
  ret = g_malloc0 (sizeof (Td_xmlfield_tmp));

  /* Champs */
  ret->name = xmlGetProp (cur, "name");
  ret->comment = xmlGetProp (cur, "comment");
  ret->key = atoi (xmlGetProp (cur, "key"));

  /* Type */
  cur = cur->childs;
  if (!strcmp (cur->name, "type"))
    {
      ret->type = xmlGetProp (cur, "name");
      ret->type_comment = xmlGetProp (cur, "comment");
    }
  return (ret);
}

/* Table */
Td_xmltable td_parse_xmltable (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
{
  xmlNodePtr cur_root = cur;
  Td_xmltable ret = NULL;
  Td_xmlfield cur_field;
  ret = g_malloc0 (sizeof (Td_xmltable_tmp));

  /* Table */
  ret->name = xmlGetProp (cur, "name");
  ret->comment = xmlGetProp (cur, "comment");

  /* Champs */
  cur = cur_root->childs->childs;
  while (cur)
    {
      if (!strcmp (cur->name, "field"))
	{
	  cur_field = td_parse_xmlfield (doc, ns, cur);
	  if (cur_field != NULL)
	    ret->fields[ret->nbfields++] = cur_field;
	  if (ret->nbfields >= 50)
	    break;
	}
      cur = cur->next;
    }
  return (ret);
}


/*****************************************************************************/
/* Vue */
/*****************************************************************************/
/* Champs */
Td_xmlviewfield td_parse_xmlviewfield (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
{
  Td_xmlviewfield ret = NULL;
  ret = g_malloc0 (sizeof (Td_xmlviewfield_tmp));

  /* Champs */
  ret->name = xmlGetProp (cur, "table_field");
  ret->plugins = xmlGetProp (cur, "name");

  /* Proprits */
  cur = cur->childs->childs;
  while (cur)
    {
      if (!strcmp (cur->name, "property"))
	{
	  ret->properties[ret->nbproperties] = xmlGetProp (cur, "name");
	  ret->properties_value[ret->nbproperties] = xmlGetProp (cur, "value");
	  ret->nbproperties++;
	  if (ret->nbproperties >= 50)
	    break;
	}
      cur = cur->next;
    }
  return (ret);
}

/* Table */
Td_xmlview td_parse_xmlview (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
{
  Td_xmlview ret = NULL;
  Td_xmlviewfield cur_viewfield;
  ret = g_malloc0 (sizeof (Td_xmlview_tmp));

  /* Vue */
  ret->name = xmlGetProp (cur, "table");
  ret->plugins = xmlGetProp (cur, "name");

  /* Champs */
  cur = cur->childs->childs;
  while (cur)
    {
      if (!strcmp (cur->name, "field"))
	{
	  cur_viewfield = td_parse_xmlviewfield (doc, ns, cur);
	  if (cur_viewfield != NULL)
	    ret->fields[ret->nbfields++] = cur_viewfield;
	  if (ret->nbfields >= 50)
	    break;
	}
      cur = cur->next;
    }
  return (ret);
}


/*****************************************************************************/
/* Table de donnes */
/*****************************************************************************/
Td_xmldatatable td_parse_xmldatatable (xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
{
  xmlNodePtr cur_root = cur;
  Td_xmldatatable ret = NULL;
  int i = 0;
  ret = g_malloc0 (sizeof (Td_xmldatatable_tmp));

  /* Table */
  ret->table_source = xmlGetProp (cur_root, "table");

  /* Items */
  cur = cur_root->childs->childs;
  ret->nbitems = 0;
  while (cur)
    {
      if (!strcmp (cur->name, "item"))
	{
	  i = 0;
	  while (xmlGetProp (cur, g_strdup_printf("field%d", i)))
	    {
	      ret->items[ret->nbitems][i] = xmlGetProp (cur, g_strdup_printf("field%d", i));
	      i++;
	    }
	  ret->nbitems++;
	  if (ret->nbitems >= 500)
	    break;
	}
      cur = cur->next;
    }
  ret->nbfields = i;
  return (ret);
}


/*****************************************************************************/
/* Base */
/*****************************************************************************/
Td_xmlbase td_parse_xmlbase (char *file)
{
  xmlDocPtr doc;
  xmlNsPtr ns;
  xmlNodePtr cur;
  Td_xmlbase ret;
  Td_xmltable cur_table;
  Td_xmlview cur_view;
  Td_xmldatatable cur_datatable;
  ret = g_malloc0 (sizeof (Td_xmlbase_tmp));

  /* Vrif du contenu du fichier */
  doc = xmlParseFile (file);
  if (!doc)
    {
      hopla_message (_("Parsing XML file : doc == NULL"), "failed");
      return (NULL);
    }	
  cur = doc->root;
  if (!cur)
    {
      hopla_message (_("Parsing XML file : cur == NULL"), "failed");
      xmlFreeDoc (doc);
      return (NULL);
    }
  ns = xmlSearchNsByHref (doc, cur, "hopla/1.0.4");
  if (!ns)
    {
      hopla_message (_("Parsing XML file : ns == NULL"), "failed");
      xmlFreeDoc (doc);
      return (NULL);
    }

  /* Base */
  cur = doc->root;
  ret->name = xmlGetProp (cur, "name");
  ret->comment = xmlGetProp (cur, "comment");

  /* Tables */
  cur = doc->root->childs->childs;
  while (cur)
    {
      if (!strcmp(cur->name, "table"))
	{
	  cur_table = td_parse_xmltable(doc, ns, cur);
	  if (cur_table != NULL)
	    ret->tables[ret->nbtables++] = cur_table;
	  if (ret->nbtables >= 50)
	    break;
	}
      cur = cur->next;
    }

  /* Vues */
  cur = doc->root->childs->next->childs;
  while (cur)
    {
      if (!strcmp (cur->name, "mod"))
	{
	  cur_view = td_parse_xmlview (doc, ns, cur);
	  if (cur_view != NULL)
	    ret->views[ret->nbviews++] = cur_view;
	  if (ret->nbviews >= 50)
	    break;
	}
      cur = cur->next;
    }

  /* Items */
  cur = doc->root->childs->next->next->childs;
  while (cur)
    {
      if (!strcmp (cur->name, "datatable"))
	{
	  cur_datatable = td_parse_xmldatatable (doc, ns, cur);
	  if (cur_datatable != NULL)
	    ret->datatables[ret->nbdatatables++] = cur_datatable;
	  if (ret->nbdatatables >= 50)
	    break;
	}
      cur = cur->next;
    }
  xmlFreeDoc (doc);
  return (ret);
}


/*****************************************************************************/
/* Enregistrer base de donnes XML */
/*****************************************************************************/
gboolean td_save_xmlbase (gchar *file, Td_xmlbase cur_base, gchar *mode)
{
  Td_xmltable cur_table;
  Td_xmlfield cur_field;
  Td_xmlview cur_view;
  Td_xmldatatable cur_datatable;
  FILE *fp;
  xmlDocPtr doc;
  xmlNsPtr ns;
  xmlNodePtr tree_tables;
  xmlNodePtr tree_table;
  xmlNodePtr tree_fields;
  xmlNodePtr tree_field;
  xmlNodePtr tree_type;
  xmlNodePtr tree_views;
  xmlNodePtr tree_view;
  xmlNodePtr tree_properties;
  xmlNodePtr tree_property;
  xmlNodePtr tree_datatables;
  xmlNodePtr tree_datatable;
  xmlNodePtr tree_items; 
  xmlNodePtr tree_item;
  int i, j, k;

  /* Vrif de l'existance du fichier */
  fp = fopen (file, "w");
  if (!fp)
    {
      hopla_message (_("Opening file"), "failed");
      return FALSE;
    }
  fclose (fp);
  doc = xmlNewDoc ("1.0");
  ns = xmlNewGlobalNs (doc, "hopla/1.0.4","hOpla" );

  /* Base */
  doc->root = xmlNewDocNode (doc, ns, "database", NULL);
  xmlSetProp (doc->root, "name", cur_base->name);
  xmlSetProp (doc->root, "comment", cur_base->comment);

  /* Tables */
  tree_tables = xmlNewChild (doc->root, ns, "tables", NULL);
  if ((!strcmp (mode, "table")) || (!strcmp (mode, "base")))
    for (i=0; i < cur_base->nbtables; i++)
      {
	cur_table = cur_base->tables[i];
	tree_table = xmlNewChild (tree_tables, ns, "table", NULL);
	xmlSetProp (tree_table, "name", cur_table->name);
	xmlSetProp (tree_table, "comment", cur_table->comment);
	tree_fields = xmlNewChild (tree_table, ns, "fields", NULL);
	for (j=0; j < cur_table->nbfields; j++)
	  {
	    cur_field = cur_table->fields[j];
	    tree_field = xmlNewChild (tree_fields, ns, "field", NULL);
	    xmlSetProp (tree_field, "name", cur_field->name);
	    xmlSetProp (tree_field, "comment", cur_field->comment);
	    xmlSetProp (tree_field, "key", g_strdup_printf ("%d", cur_field->key));
	    tree_type = xmlNewChild (tree_field, ns, "type", NULL);
	    xmlSetProp (tree_type, "name", cur_field->type);
	    xmlSetProp (tree_type, "comment", cur_field->type_comment);
	  }
      }

  /* Vues */
  tree_views = xmlNewChild (doc->root, ns, "mods", NULL);
  if ((!strcmp (mode, "view")) || (!strcmp (mode, "base")))
    {
      for (i=0; i<cur_base->nbviews; i++)
	{
	  tree_view = xmlNewChild (tree_views, ns, "mod", NULL);
	  xmlSetProp (tree_view, "name", cur_base->views[i]->plugins);
	  xmlSetProp (tree_view, "table", cur_base->views[i]->name);
	  tree_fields = xmlNewChild (tree_view, ns, "fields", NULL);
	  for (j=0; j<cur_base->views[i]->nbfields; j++)
	    {
	      tree_field = xmlNewChild (tree_fields, ns, "field", NULL);
	      xmlSetProp (tree_field, "name", cur_base->views[i]->fields[j]->plugins);
	      xmlSetProp (tree_field, "table_field", cur_base->views[i]->fields[j]->name);
	      tree_properties = xmlNewChild (tree_field, ns, "properties", NULL);
	      for (k=0; k<cur_base->views[i]->fields[j]->nbproperties; k++)
		{
		  tree_property = xmlNewChild (tree_properties, ns, "property", NULL);
		  xmlSetProp (tree_property, "name", cur_base->views[i]->fields[j]->properties[k]);
		  xmlSetProp (tree_property, "value", cur_base->views[i]->fields[j]->properties_value[k]);
		}
	    }
	}
    }

  /* Table de donnes */
  tree_datatables = xmlNewChild (doc->root, ns, "datatables", NULL);
  if ((!strcmp (mode, "datatable")) || (!strcmp (mode, "base")))
    for (i=0; i < cur_base->nbdatatables; i++)
      {
	cur_datatable = cur_base->datatables[i];
	tree_datatable = xmlNewChild (tree_datatables, ns, "datatable", NULL);
	xmlSetProp (tree_datatable, "table", cur_datatable->table_source);
	tree_items = xmlNewChild (tree_datatable, ns, "items", NULL);
	for (j=0; j<cur_datatable->nbitems; j++)
	  {
	    tree_item = xmlNewChild (tree_items, ns, "item", NULL);
	    for (k=0; k<cur_datatable->nbfields; k++)
	      xmlSetProp (tree_item, g_strdup_printf ("field%d", k), cur_datatable->items[j][k]);
	  }
      }

  /* Fin */
  if (xmlSaveFile (file, doc) < 0)
    {
      hopla_message (_("Writting file"), "failed");
      xmlFreeDoc (doc);
      return FALSE;
    }
  xmlFreeDoc (doc);
  return TRUE;
}
